@@ -24,7 +24,7 @@ pub fn max_non_leaf_size(circuit: CircuitRc) -> BigUint {
         .chain(once(if circuit.is_leaf() {
             BIG_UINT_0.clone()
         } else {
-            circuit.info().numel()
+            circuit.info().naive_mem_use(None)
         }))
         .max()
         .unwrap_or(BIG_UINT_0.clone());
