@@ -7,12 +7,14 @@ use num_bigint::BigUint;
 use pyo3::{prelude::*, types::PyTuple};
 use rr_util::{
     atr,
+    compact_data::U8Set,
     rearrange_spec::RInnerInts,
     tensor_util::{IndexError, IndexTensor, TensorIndexSync},
     tu8v,
     util::EinsumAxes,
 };
 use rr_util::{
+    name::Name,
     opt_einsum::EinsumSpec,
     py_types::{
         einops_repeat, einsum_py, make_diagonal_py, scalar_to_tensor, scalar_to_tensor_py,
@@ -53,15 +55,13 @@ pub struct Einsum {
     pub args: EinsumArgs,
     pub out_axes: EinsumAxes,
     info: CachedCircuitInfo,
-    name: Option<String>,
 }
 
 circuit_node_extra_impl!(Einsum);
 
 impl CircuitNodeComputeInfoImpl for Einsum {
     fn compute_shape(&self) -> Shape {
-        let map = self.shape_map().unwrap();
-        self.out_axes.iter().map(|a| *map.get(a).unwrap()).collect()
+        self.info().shape.clone()
     }
 }
 impl CircuitNodeHashWithChildren for Einsum {
@@ -122,7 +122,7 @@ impl CircuitNode for Einsum {
                 })
                 .collect::<Result<_, _>>()?,
             self.out_axes.clone(),
-            self.name.clone(),
+            self.info().name,
         )
     }
 
@@ -173,7 +173,7 @@ impl Einsum {
                 bail!(ConstructError::EinsumLenShapeDifferentFromAxes {
                     len_axes: axes.len(),
                     child_len_shape: circ.info().rank(),
-                    einsum_name: self.name_cloned(),
+                    einsum_name: self.info().name,
                     child_circuit: circ.clone(),
                 });
             }
@@ -184,7 +184,7 @@ impl Einsum {
                         new_size: circuit_size.into(),
                         existing_size: existing_size.into(),
                         axis: *axis as usize,
-                        einsum_name: self.name_cloned(),
+                        einsum_name: self.info().name,
                         child_circuit: circ.clone(),
                     });
                 }
@@ -195,36 +195,36 @@ impl Einsum {
     }
 
     #[apply(new_rc_unwrap)]
-    pub fn try_new(args: EinsumArgs, out_axes: EinsumAxes, name: Option<String>) -> Result<Self> {
+    pub fn try_new(args: EinsumArgs, out_axes: EinsumAxes, name: Option<Name>) -> Result<Self> {
         // TODO: reduce axes nums!
-        let out = Self {
+        let mut out = Self {
             args,
             out_axes,
-            name,
             info: Default::default(),
         };
-
+        out.info.name = name;
         let map = out.shape_map()?; // catch errors
 
-        for a in &out.out_axes {
-            if !map.contains_key(a) {
-                return Err(ConstructError::EinsumOutputNotSubset {
-                    circuit_name: out.name.clone(),
+        out.info.shape = out
+            .out_axes
+            .iter()
+            .map(|a| map.get(a).cloned())
+            .collect::<Option<Shape>>()
+            .ok_or_else(|| {
+                anyhow::Error::from(ConstructError::EinsumOutputNotSubset {
+                    circuit_name: out.info().name,
                     all_input_axes: map.into_keys().collect(),
                     output_axes: out.out_axes.clone(),
-                }
-                .into());
-            }
-        }
+                })
+            })?;
 
-        // could reuse hashmap if helped with perf
         out.initial_init_info()
     }
 
     pub fn try_from_spec(
         spec: &EinsumSpec,
         circuits: &[CircuitRc],
-        name: Option<String>,
+        name: Option<Name>,
     ) -> Result<Self> {
         // TODO: maybe check int sizes!
         assert_eq!(spec.input_ints.len(), circuits.len());
@@ -250,13 +250,14 @@ impl Einsum {
 impl CircuitNodeAutoName for Einsum {
     const PRIORITY: OperatorPriority = OperatorPriority::Infix { priority: 1 };
 
-    fn auto_name(&self) -> Option<String> {
+    fn auto_name(&self) -> Option<Name> {
         children_names_with_maybe_paren(&Self::PRIORITY, self.children().collect()).map(|names| {
             names
                 .iter()
                 .map(|name| Self::shorten_child_name(name))
                 .collect::<Vec<String>>()
                 .join(" * ")
+                .into()
         })
     }
 }
@@ -285,7 +286,7 @@ impl Einsum {
     fn new_py(
         args: Vec<(CircuitRc, EinsumAxes)>,
         out_axes: EinsumAxes,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> PyResult<PyClassInitializer<Self>> {
         let out = Self::try_new(args, out_axes, name)?;
 
@@ -315,7 +316,7 @@ impl Einsum {
         nodes: Vec<CircuitRc>,
         input_ints: Vec<EinsumAxes>,
         output_ints: EinsumAxes,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> Result<Self> {
         if nodes.len() != input_ints.len() {
             bail!(ConstructError::EinsumWrongNumChildren {
@@ -332,7 +333,7 @@ impl Einsum {
     pub fn from_einsum_string(
         string: &str,
         nodes: Vec<CircuitRc>,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> Result<Self> {
         let (mut input_ints, output_ints) = EinsumSpec::string_to_ints(string.to_owned())?;
         if input_ints.len() == 1 && input_ints[0].is_empty() && nodes.len() == 0 {
@@ -346,7 +347,7 @@ impl Einsum {
     pub fn from_fancy_string(
         string: &str,
         nodes: Vec<CircuitRc>,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> Result<Self> {
         let (mut input_ints, output_ints) = EinsumSpec::fancy_string_to_ints(string.to_owned())?;
         if input_ints.len() == 1 && input_ints[0].is_empty() && nodes.len() == 0 {
@@ -360,19 +361,19 @@ impl Einsum {
     pub fn from_spec_py(
         spec: EinsumSpec,
         nodes: Vec<CircuitRc>,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> Result<Self> {
         Self::try_from_spec(&spec, &nodes, name)
     }
 
     #[staticmethod]
-    pub fn new_diag(node: CircuitRc, ints: EinsumAxes, name: Option<String>) -> Self {
+    pub fn new_diag(node: CircuitRc, ints: EinsumAxes, name: Option<Name>) -> Self {
         let deduped = ints.iter().unique().cloned().collect();
         Einsum::try_new(vec![(node, deduped)], ints, name).unwrap()
     }
 
     #[staticmethod]
-    pub fn new_trace(node: CircuitRc, ints: EinsumAxes, name: Option<String>) -> Self {
+    pub fn new_trace(node: CircuitRc, ints: EinsumAxes, name: Option<Name>) -> Self {
         let deduped = ints.iter().unique().cloned().collect();
         Einsum::try_new(vec![(node, ints)], deduped, name).unwrap()
     }
@@ -381,8 +382,8 @@ impl Einsum {
     pub fn scalar_mul(
         node: CircuitRc,
         scalar: f64,
-        name: Option<String>,
-        scalar_name: Option<String>,
+        name: Option<Name>,
+        scalar_name: Option<Name>,
     ) -> Self {
         let axes: EinsumAxes = (0..node.info().rank()).map(|x| x as u8).collect();
         Einsum::try_new(
@@ -397,7 +398,7 @@ impl Einsum {
     }
     #[staticmethod]
     #[args(nodes = "*", name = "None")]
-    pub fn elementwise_broadcasted(nodes: Vec<CircuitRc>, name: Option<String>) -> Result<Einsum> {
+    pub fn elementwise_broadcasted(nodes: Vec<CircuitRc>, name: Option<Name>) -> Result<Einsum> {
         let out_shape = broadcast_shapes(
             &nodes
                 .iter()
@@ -434,12 +435,12 @@ impl Einsum {
         )
     }
     #[staticmethod]
-    pub fn empty(name: Option<String>) -> Self {
+    pub fn empty(name: Option<Name>) -> Self {
         Einsum::try_new(vec![], tu8v![], name).unwrap()
     }
 
     #[staticmethod]
-    pub fn identity(node: CircuitRc, name: Option<String>) -> Self {
+    pub fn identity(node: CircuitRc, name: Option<Name>) -> Self {
         let axes: EinsumAxes = (0..node.info().rank()).map(|x| x as u8).collect();
         Einsum::try_new(vec![(node, axes.clone())], axes, name).unwrap()
     }
@@ -447,7 +448,7 @@ impl Einsum {
     #[staticmethod]
     pub fn new_outer_product(
         nodes: Vec<CircuitRc>,
-        name: Option<String>,
+        name: Option<Name>,
         out_axes_permutation: Option<Vec<usize>>,
     ) -> Self {
         let sections: Vec<_> = nodes.iter().map(|n| n.info().rank()).collect();
@@ -482,12 +483,12 @@ impl Einsum {
         Einsum::try_new(
             args.unwrap_or_else(|| self.args.clone()),
             out_axes.unwrap_or_else(|| self.out_axes.clone()),
-            self.name.clone(),
+            self.info().name,
         )
         .unwrap()
     }
 
-    pub fn reduced_axes(&self) -> HashSet<u8> {
+    pub fn reduced_axes(&self) -> U8Set {
         self.input_axes()
             .flatten()
             .filter(|i| !self.out_axes.contains(i))
@@ -526,7 +527,7 @@ impl Einsum {
         Einsum::try_from_spec(
             &spec_normalized,
             &self.input_circuits().collect::<Vec<CircuitRc>>(),
-            self.name_cloned(),
+            self.info().name,
         )
         .unwrap()
     }
@@ -539,27 +540,23 @@ pub struct Add {
     #[pyo3(get)]
     pub nodes: Vec<CircuitRc>,
     info: CachedCircuitInfo,
-    name: Option<String>,
 }
 
 impl Add {
     #[apply(new_rc_unwrap)]
-    pub fn try_new(nodes: Vec<CircuitRc>, name: Option<String>) -> Result<Self> {
+    pub fn try_new(nodes: Vec<CircuitRc>, name: Option<Name>) -> Result<Self> {
         // TODO: reduce axes nums!
-        let out = Self {
+        let mut out = Self {
             nodes,
-            name,
             info: Default::default(),
         };
+        out.info.name = name;
         out.compute_shape_maybe()
-            .with_context(|| format!("Sum isn't broadcastable name={:?}", out.name()))?;
+            .with_context(|| format!("Sum isn't broadcastable name={:?}", out.info().name))?;
         out.initial_init_info()
     }
 
-    pub fn try_from_counts(
-        nodes: &HashMap<CircuitRc, usize>,
-        name: Option<String>,
-    ) -> Result<Self> {
+    pub fn try_from_counts(nodes: &HashMap<CircuitRc, usize>, name: Option<Name>) -> Result<Self> {
         Self::try_new(
             nodes
                 .iter()
@@ -584,7 +581,7 @@ impl Add {
 impl Add {
     #[new]
     #[args(nodes = "*", name = "None")]
-    fn new_py(nodes: Vec<CircuitRc>, name: Option<String>) -> PyResult<PyClassInitializer<Self>> {
+    fn new_py(nodes: Vec<CircuitRc>, name: Option<Name>) -> PyResult<PyClassInitializer<Self>> {
         let out = Self::try_new(nodes, name)?;
 
         Ok(out.into_init())
@@ -619,7 +616,7 @@ impl Add {
     pub fn from_weighted_nodes(
         nodes_and_weights: Vec<(CircuitRc, f64)>,
         use_1_weights: bool,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> Result<Self> {
         let children = nodes_and_weights
             .iter()
@@ -635,7 +632,7 @@ impl Add {
     }
 
     #[staticmethod]
-    pub fn minus(positive: CircuitRc, negative: CircuitRc, name: Option<String>) -> Result<Self> {
+    pub fn minus(positive: CircuitRc, negative: CircuitRc, name: Option<Name>) -> Result<Self> {
         positive.sub(negative, name)
     }
 }
@@ -687,7 +684,7 @@ impl CircuitNode for Add {
                 .enumerate()
                 .map(move |(i, circ)| f(i, circ.clone()))
                 .collect::<Result<Vec<_>, _>>()?,
-            self.name.clone(),
+            self.info().name,
         )
     }
 
@@ -712,13 +709,14 @@ impl CircuitNode for Add {
 impl CircuitNodeAutoName for Add {
     const PRIORITY: OperatorPriority = OperatorPriority::Infix { priority: 0 };
 
-    fn auto_name(&self) -> Option<String> {
+    fn auto_name(&self) -> Option<Name> {
         children_names_with_maybe_paren(&Self::PRIORITY, self.children().collect()).map(|names| {
             names
                 .iter()
                 .map(|name| Self::shorten_child_name(name))
                 .collect::<Vec<String>>()
                 .join(" + ")
+                .into()
         })
     }
 }
@@ -731,20 +729,19 @@ pub struct Rearrange {
     #[pyo3(get)]
     pub spec: Box<RearrangeSpec>,
     info: CachedCircuitInfo,
-    name: Option<String>,
 }
 
 impl Rearrange {
     #[apply(new_rc_unwrap)]
-    pub fn try_new(node: CircuitRc, spec: RearrangeSpec, name: Option<String>) -> Result<Self> {
+    pub fn try_new(node: CircuitRc, spec: RearrangeSpec, name: Option<Name>) -> Result<Self> {
         // spec must be valid because we enforce that on construction!
         // TODO: reduce axes nums!
-        let out = Self {
+        let mut out = Self {
             node: node.clone(),
             spec: Box::new(spec),
-            name,
             info: Default::default(),
         };
+        out.info.name = name;
 
         // Check that RearrangeSpec is valid for input, prevent panic on rearrange.info().shape
         out.spec
@@ -753,7 +750,7 @@ impl Rearrange {
                 format!(
                     "{}\nname={:?}, child={:?}",
                     "rearrange spec conform_to_input_shape failed in construct rearrange",
-                    out.name(),
+                    out.info().name,
                     node
                 )
             })?;
@@ -798,7 +795,7 @@ impl Rearrange {
         Rearrange::try_new(
             node.unwrap_or_else(|| self.node.clone()),
             spec.unwrap_or_else(|| (*self.spec).clone()),
-            self.name.clone(),
+            self.info().name,
         )
         .unwrap()
     }
@@ -806,7 +803,7 @@ impl Rearrange {
     pub fn nrc_elim_identity(
         node: CircuitRc,
         spec: RearrangeSpec,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> CircuitRc {
         if spec.is_identity() {
             node
@@ -815,7 +812,7 @@ impl Rearrange {
         }
     }
 
-    pub fn unflatten(node: CircuitRc, shape: Shape, name: Option<String>) -> Result<Self> {
+    pub fn unflatten(node: CircuitRc, shape: Shape, name: Option<Name>) -> Result<Self> {
         if node.ndim() != 1 {
             bail!(ConstructError::UnflattenButNDimNot1 { ndim: node.ndim() })
         }
@@ -865,7 +862,7 @@ impl CircuitNode for Rearrange {
         Self::try_new(
             f(0, self.node.clone())?,
             (*self.spec).clone(),
-            self.name.clone(),
+            self.info().name,
         )
     }
 
@@ -881,8 +878,9 @@ impl CircuitNode for Rearrange {
 impl CircuitNodeAutoName for Rearrange {
     const PRIORITY: OperatorPriority = OperatorPriority::PostFix {};
 
-    fn auto_name(&self) -> Option<String> {
-        child_name_with_maybe_paren(&Self::PRIORITY, self.node.clone()).map(|n| n + " rearrange")
+    fn auto_name(&self) -> Option<Name> {
+        child_name_with_maybe_paren(&Self::PRIORITY, self.node.clone())
+            .map(|n| (n.string() + " rearrange").into())
     }
 }
 
@@ -892,7 +890,7 @@ impl Rearrange {
     fn new_py(
         node: CircuitRc,
         spec: RearrangeSpec,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> PyResult<PyClassInitializer<Self>> {
         let out = Rearrange::try_new(node, spec, name)?;
 
@@ -900,7 +898,7 @@ impl Rearrange {
     }
 
     #[staticmethod]
-    pub fn from_string(node: CircuitRc, string: &str, name: Option<String>) -> Result<Self> {
+    pub fn from_string(node: CircuitRc, string: &str, name: Option<Name>) -> Result<Self> {
         Rearrange::try_new(
             node,
             string
@@ -945,12 +943,11 @@ pub struct Index {
     #[pyo3(get, name = "idx")]
     pub index: TensorIndex,
     info: CachedCircuitInfo,
-    name: Option<String>,
 }
 
 impl Index {
     #[apply(new_rc_unwrap)]
-    pub fn try_new(node: CircuitRc, index: TensorIndex, name: Option<String>) -> Result<Self> {
+    pub fn try_new(node: CircuitRc, index: TensorIndex, name: Option<Name>) -> Result<Self> {
         let mut index = index;
         let node_rank = node.info().rank();
         match index.0.len().cmp(&node_rank) {
@@ -985,12 +982,13 @@ impl Index {
 
         index.validate(&node.info().shape)?;
 
-        let out = Self {
+        let mut out = Self {
             node,
             index,
-            name,
             info: Default::default(),
         };
+        out.info.name = name;
+
         out.initial_init_info()
     }
 }
@@ -1051,7 +1049,7 @@ impl CircuitNode for Index {
         Self::try_new(
             f(0, self.node.clone())?,
             self.index.clone(),
-            self.name.clone(),
+            self.info().name,
         )
     }
 
@@ -1074,8 +1072,9 @@ impl CircuitNode for Index {
 impl CircuitNodeAutoName for Index {
     const PRIORITY: OperatorPriority = OperatorPriority::PostFix {};
 
-    fn auto_name(&self) -> Option<String> {
-        child_name_with_maybe_paren(&Self::PRIORITY, self.node.clone()).map(|n| n + " idx")
+    fn auto_name(&self) -> Option<Name> {
+        child_name_with_maybe_paren(&Self::PRIORITY, self.node.clone())
+            .map(|n| (n.string() + " idx").into())
     }
 }
 
@@ -1085,7 +1084,7 @@ impl Index {
     fn new_py(
         node: CircuitRc,
         index: TensorIndex,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> PyResult<PyClassInitializer<Index>> {
         let out = Index::try_new(node, index, name)?;
 
@@ -1115,7 +1114,7 @@ impl Index {
                     })
                     .collect(),
             ),
-            self.name_cloned(),
+            self.info().name,
         )
         .unwrap()
     }
@@ -1123,7 +1122,7 @@ impl Index {
     pub fn new_synchronized_to_start(
         node: CircuitRc,
         index: TensorIndexSync, // tensor index
-        name: Option<String>,
+        name: Option<Name>,
     ) -> Result<Self> {
         let index: TensorIndex = index.0;
         // let broadcast_shape = broadc
@@ -1149,7 +1148,7 @@ impl Index {
                     .collect(),
                 int_sizes: sv![OpSize::NONE;index.0.len()],
             },
-            name.as_ref().map(|x| format!("{}_pre_idx_flat", x)),
+            name.as_ref().map(|x| format!("{}_pre_idx_flat", x).into()),
         )?
         .rc();
         let tensors_index = TensorIndex(
@@ -1192,7 +1191,7 @@ impl Index {
         let indexed = Index::try_new(
             rearranged_tensors_idxs_first,
             TensorIndex(vec![TensorAxisIndex::Tensor(broadcasted_tensor)]),
-            name.as_ref().map(|x| format!("{}_t_idx", x)),
+            name.as_ref().map(|x| format!("{}_t_idx", x).into()),
         )?
         .rc();
         let broadcasted_shape = broadcast_shapes(
@@ -1224,7 +1223,7 @@ impl Index {
                     .chain(nontensor_part_is.iter().map(|_| OpSize::NONE))
                     .collect(),
             },
-            name.as_ref().map(|x| format!("{}_post_t_shape", x)),
+            name.as_ref().map(|x| format!("{}_post_t_shape", x).into()),
         )?
         .rc();
         let nontensor_index: Vec<TensorAxisIndex> = nontensor_part_is
@@ -1278,12 +1277,11 @@ pub struct Concat {
     pub nodes: Vec<CircuitRc>,
     pub axis: usize,
     info: CachedCircuitInfo,
-    name: Option<String>,
 }
 
 impl Concat {
     #[apply(new_rc_unwrap)]
-    pub fn try_new(nodes: Vec<CircuitRc>, axis: usize, name: Option<String>) -> Result<Self> {
+    pub fn try_new(nodes: Vec<CircuitRc>, axis: usize, name: Option<Name>) -> Result<Self> {
         if nodes.is_empty() {
             return Err(ConstructError::ConcatZeroNodes {}.into());
         }
@@ -1305,12 +1303,12 @@ impl Concat {
             }
         }
 
-        let out = Self {
+        let mut out = Self {
             nodes,
             axis,
             info: Default::default(),
-            name,
         };
+        out.info.name = name;
         out.initial_init_info()
     }
 
@@ -1357,7 +1355,7 @@ impl CircuitNode for Concat {
                 .map(move |(i, circ)| f(i, circ.clone()))
                 .collect::<Result<Vec<_>, _>>()?,
             self.axis,
-            self.name.clone(),
+            self.info().name,
         )
     }
 
@@ -1389,13 +1387,14 @@ impl CircuitNode for Concat {
 impl CircuitNodeAutoName for Concat {
     const PRIORITY: OperatorPriority = OperatorPriority::InfixAmbiguous {};
 
-    fn auto_name(&self) -> Option<String> {
+    fn auto_name(&self) -> Option<Name> {
         children_names_with_maybe_paren(&Self::PRIORITY, self.children().collect()).map(|names| {
             names
                 .iter()
                 .map(|name| Self::shorten_child_name(name))
                 .collect::<Vec<String>>()
                 .join(" ++ ")
+                .into()
         })
     }
 }
@@ -1407,10 +1406,10 @@ impl Concat {
     fn new_py(
         nodes: Vec<CircuitRc>,
         axis: i64,
-        name: Option<&str>,
+        name: Option<Name>,
     ) -> Result<PyClassInitializer<Concat>> {
         let axis = Self::convert_axis(&nodes, axis, false)?;
-        let out = Self::try_new(nodes, axis, name.map(str::to_owned))?;
+        let out = Self::try_new(nodes, axis, name)?;
         Ok(out.into_init())
     }
 
@@ -1433,14 +1432,16 @@ impl Concat {
 
     #[staticmethod]
     #[args(nodes = "*", axis, name = "None")]
-    pub fn stack(nodes: Vec<CircuitRc>, axis: i64, name: Option<String>) -> Result<Self> {
+    pub fn stack(nodes: Vec<CircuitRc>, axis: i64, name: Option<Name>) -> Result<Self> {
         let axis = Self::convert_axis(&nodes, axis, true)?;
         let to_cat = nodes
             .into_iter()
             .map(|c| {
                 c.unsqueeze(
                     vec![axis],
-                    c.name().map(|s| format!("{}_unsqueeze_stack", s)),
+                    c.info()
+                        .name
+                        .map(|s| format!("{}_unsqueeze_stack", s).into()),
                 )
                 .map(|x| x.rc())
             })
@@ -1464,7 +1465,6 @@ pub struct Scatter {
     #[pyo3(get, name = "idx")]
     pub index: TensorIndex,
     info: CachedCircuitInfo,
-    name: Option<String>,
 }
 
 impl Scatter {
@@ -1473,7 +1473,7 @@ impl Scatter {
         node: CircuitRc,
         index: TensorIndex,
         shape: Shape,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> Result<Self> {
         let index_shape = index.apply_to_shape(&shape);
         if index.all_uslices().is_none() {
@@ -1491,9 +1491,9 @@ impl Scatter {
         let mut out = Self {
             node,
             index,
-            name,
             info: Default::default(),
         };
+        out.info.name = name;
         out.info.shape = shape;
         // todo check not too many axes / oob
         out.initial_init_info()
@@ -1549,7 +1549,7 @@ impl CircuitNode for Scatter {
             f(0, self.node.clone())?,
             self.index.clone(),
             self.info().shape.clone(),
-            self.name.clone(),
+            self.info().name,
         )
     }
 
@@ -1571,8 +1571,9 @@ impl CircuitNode for Scatter {
 impl CircuitNodeAutoName for Scatter {
     const PRIORITY: OperatorPriority = OperatorPriority::PostFix {};
 
-    fn auto_name(&self) -> Option<String> {
-        child_name_with_maybe_paren(&Self::PRIORITY, self.node.clone()).map(|n| n + " scatter")
+    fn auto_name(&self) -> Option<Name> {
+        child_name_with_maybe_paren(&Self::PRIORITY, self.node.clone())
+            .map(|n| (n.string() + " scatter").into())
     }
 }
 
@@ -1583,7 +1584,7 @@ impl Scatter {
         node: CircuitRc,
         index: TensorIndex,
         shape: Shape,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> PyResult<PyClassInitializer<Scatter>> {
         let out = Scatter::try_new(node, index, shape, name)?;
 
@@ -1603,13 +1604,13 @@ pub fn flat_concat(circuits: Vec<CircuitRc>) -> Result<Concat> {
             Ok(Rearrange::try_new(
                 x.clone(),
                 RearrangeSpec::flatten_usize(x.info().rank())?,
-                Some("flatten".to_owned()),
+                Some("flatten".into()),
             )
             .unwrap()
             .rc())
         })
         .collect::<Result<_>>()?;
-    Ok(Concat::try_new(flatteneds, 0, Some("flat_concat".to_owned())).unwrap())
+    Ok(Concat::try_new(flatteneds, 0, Some("flat_concat".into())).unwrap())
 }
 
 #[pyfunction]
@@ -1655,7 +1656,6 @@ pub struct Conv {
     pub padding: Vec<(usize, usize)>,
 
     info: CachedCircuitInfo,
-    name: Option<String>,
 }
 
 impl Conv {
@@ -1665,7 +1665,7 @@ impl Conv {
         filter: CircuitRc,
         stride: Vec<usize>,
         padding: Vec<(usize, usize)>,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> Result<Self> {
         let dims = stride.len();
         if stride.len() != padding.len() {
@@ -1676,7 +1676,7 @@ impl Conv {
         }
         if dims > 3 {
             bail!(ConstructError::NotSupportedYet {
-                string: "Conv over more than 3 dims not supported".to_owned(),
+                string: "Conv over more than 3 dims not supported".into(),
             });
         }
 
@@ -1685,9 +1685,9 @@ impl Conv {
             filter,
             stride,
             padding,
-            name,
             info: Default::default(),
         };
+        out.info.name = name;
         out.info.shape = out.try_compute_shape()?;
         // todo check not too many axes / oob
         out.initial_init_info()
@@ -1822,7 +1822,7 @@ impl CircuitNode for Conv {
             f(1, self.filter.clone())?,
             self.stride.clone(),
             self.padding.clone(),
-            self.name.clone(),
+            self.info().name,
         )
     }
 
@@ -1844,10 +1844,10 @@ impl CircuitNode for Conv {
 impl CircuitNodeAutoName for Conv {
     const PRIORITY: OperatorPriority = OperatorPriority::Function {};
 
-    fn auto_name(&self) -> Option<String> {
-        let input_name = self.input.name()?;
-        let filter_name = self.filter.name()?;
-        Some(format!("Conv({input_name}, {filter_name})"))
+    fn auto_name(&self) -> Option<Name> {
+        let input_name = self.input.info().name?;
+        let filter_name = self.filter.info().name?;
+        Some(format!("Conv({input_name}, {filter_name})").into())
     }
 }
 
@@ -1877,7 +1877,7 @@ impl Conv {
         filter: CircuitRc,
         stride: Vec<usize>,
         padding: ConvPaddingShorthand,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> PyResult<PyClassInitializer<Conv>> {
         let padding = padding.expand(stride.len());
         let out = Conv::try_new(input, filter, stride, padding, name)?;
