@@ -8,6 +8,7 @@ use pyo3::{exceptions::PyValueError, prelude::*};
 use regex::Regex;
 use rr_util::{
     lru_cache::TensorCacheRrfs,
+    name::Name,
     python_error_exception,
     symbolic_size::{SymbolicSizeProduct, SIZE_PROD_MATCH},
     tensor_db::ensure_all_tensors_local,
@@ -33,7 +34,7 @@ const FANCY_PREFIX: &str = "fancy:";
 #[derive(Clone, Debug)]
 pub struct Parser {
     #[pyo3(get, set)]
-    pub reference_circuits: HashMap<String, CircuitRc>,
+    pub reference_circuits: HashMap<Name, CircuitRc>,
     #[pyo3(get, set)]
     pub tensors_as_random: bool,
     #[pyo3(get, set)]
@@ -63,14 +64,15 @@ impl Default for Parser {
 }
 
 pub(super) fn get_reference_circuits(
-    reference_circuits: HashMap<String, CircuitRc>,
+    reference_circuits: HashMap<Name, CircuitRc>,
     reference_circuits_by_name: Vec<CircuitRc>,
-) -> Result<HashMap<String, CircuitRc>> {
+) -> Result<HashMap<Name, CircuitRc>> {
     let ref_circ_names = reference_circuits_by_name
         .into_iter()
         .map(|circ| {
             Ok((
-                circ.name_cloned()
+                circ.info()
+                    .name
                     .ok_or_else(|| ReferenceCircError::ByNameHasNoneName {
                         circuit: circ.clone(),
                     })?,
@@ -110,7 +112,7 @@ impl Parser {
     )]
     #[new]
     pub fn new(
-        reference_circuits: HashMap<String, CircuitRc>,
+        reference_circuits: HashMap<Name, CircuitRc>,
         reference_circuits_by_name: Vec<CircuitRc>,
         tensors_as_random: bool,
         tensors_as_random_device_dtype: TorchDeviceDtypeOp,
@@ -192,7 +194,7 @@ struct PartialCirc {
     pub variant: CircuitType,
     pub extra: String,
     pub shape: Option<Shape>,
-    pub name: Option<String>,
+    pub name: Option<Name>,
     pub children: Vec<CircuitIdent>,
     pub named_axes: Option<NamedAxes>,
     pub module_arg_specs: Vec<ModuleArgSpec>,
@@ -368,7 +370,7 @@ impl Parser {
 
             let is_ref = circuit_ident
                 .as_str()
-                .map(|s| self.reference_circuits.contains_key(s))
+                .map(|s| self.reference_circuits.contains_key(&Name::new(s)))
                 .unwrap_or(false);
             if is_ref {
                 if re_captures.get(name_cap).is_some()
@@ -385,14 +387,18 @@ impl Parser {
                     );
                 }
             } else {
-                let name = re_captures.get(name_cap).map(unescape_name).or_else(|| {
-                    if is_new_node {
-                        circuit_ident.as_str().map(str::to_owned)
-                    } else {
-                        // if not new node, than this is just for checking and we want to avoid checking in this case
-                        None
-                    }
-                });
+                let name = re_captures
+                    .get(name_cap)
+                    .map(unescape_name)
+                    .map(|z| Name::new(&z))
+                    .or_else(|| {
+                        if is_new_node {
+                            circuit_ident.as_str().map(|z| Name::new(z))
+                        } else {
+                            // if not new node, than this is just for checking and we want to avoid checking in this case
+                            None
+                        }
+                    });
 
                 let (shape, named_axes) = if let Some(shape_cap) = re_captures.get(shape_cap) {
                     let mut axis_strs: Vec<_> =
@@ -428,7 +434,7 @@ impl Parser {
                     let mut named_axes = NamedAxes::default();
                     for (i, (_, name)) in axes_info.iter().enumerate() {
                         if let Some(name_) = name {
-                            named_axes.insert(i.try_into().unwrap(), (*name_).to_owned());
+                            named_axes.insert(i.try_into().unwrap(), Name::new(*name_));
                         }
                     }
 
@@ -446,7 +452,7 @@ impl Parser {
                     partial_circuits.insert(
                         circuit_ident.clone(),
                         PartialCirc {
-                            name,
+                            name: name,
                             shape,
                             variant: variant.ok_or_else(|| {
                                 ParseCircuitError::RegexDidntMatchGroup {
@@ -486,7 +492,6 @@ impl Parser {
                             fail_check!($n, |x| x)
                         };
                     }
-
                     fail_check!(name, Some);
                     fail_check!(shape, Some);
                     fail_check!(variant);
@@ -509,7 +514,7 @@ impl Parser {
             }
             if let Some((l, is_new)) = stack.last() {
                 if let Str(s) = l {
-                    if self.reference_circuits.contains_key(s) {
+                    if self.reference_circuits.contains_key(&Name::new(&s)) {
                         bail!(ParseCircuitError::ReferenceCircuitHasChildren {
                             reference_name: s.to_owned(),
                             child_line: get_unindented_line(),
@@ -700,7 +705,7 @@ impl Parser {
     ) -> Result<CircuitRc> {
         if let Some(ref_circ) = ident
             .as_str()
-            .map(|x| self.reference_circuits.get(x))
+            .map(|x| self.reference_circuits.get(&Name::new(x)))
             .flatten()
         {
             return Ok(ref_circ.clone());
@@ -739,7 +744,7 @@ impl Parser {
                     ident, ps.variant, ps
                 )
             })?
-            .rename(partial_circuits[ident].name.clone());
+            .rename(partial_circuits[ident].name.into());
 
         context.insert(ident.clone(), result.clone());
         in_progress.remove(ident);
@@ -795,12 +800,12 @@ impl Parser {
                         ps.shape.clone().ok_or(ParseCircuitError::ShapeNeeded {
                             variant: ps.variant,
                         })?,
-                        ps.name.clone(),
+                        ps.name,
                         self.tensors_as_random_device_dtype.clone(),
                     )
                     .rc()
                 } else {
-                    let out = Array::from_hash_prefix(ps.name.clone(), &ps.extra, tensor_cache)
+                    let out = Array::from_hash_prefix(ps.name, &ps.extra, tensor_cache)
                         .context("failed to parse array constant from hash prefix")?
                         .rc();
                     if let Some(shape) = &ps.shape {
@@ -824,41 +829,35 @@ impl Parser {
                             variant: ps.variant,
                         })?
                         .clone(),
-                    ps.name.clone(),
+                    ps.name,
                 )
             }
             T::Add => {
                 extra_should_be_empty()?;
-                Add::try_new(children, ps.name.clone())?.rc()
+                Add::try_new(children, ps.name)?.rc()
             }
             T::Concat => Concat::try_new(
                 children,
                 parse_numeric(&ps.extra).context("failed to parse out concat axis")?,
-                ps.name.clone(),
+                ps.name,
             )?
             .rc(),
             T::Einsum => if ps.extra.starts_with(FANCY_PREFIX) {
-                Einsum::from_fancy_string(
-                    &ps.extra[FANCY_PREFIX.len()..],
-                    children,
-                    ps.name.clone(),
-                )
-                .context(concat!(
-                    "Couldn't parse fancy einsum string. If you wanted ",
-                    "to parse a regular einsum, remove \"fancy:\"."
-                ))
+                Einsum::from_fancy_string(&ps.extra[FANCY_PREFIX.len()..], children, ps.name)
+                    .context(concat!(
+                        "Couldn't parse fancy einsum string. If you wanted ",
+                        "to parse a regular einsum, remove \"fancy:\"."
+                    ))
             } else {
-                Einsum::from_einsum_string(&ps.extra, children.clone(), ps.name.clone()).context(
-                    concat!(
-                        "Couldn't parse einsum string. If you wanted ",
-                        "to parse a fancy einsum string, prefix it by \"fancy:\"."
-                    ),
-                )
+                Einsum::from_einsum_string(&ps.extra, children.clone(), ps.name).context(concat!(
+                    "Couldn't parse einsum string. If you wanted ",
+                    "to parse a fancy einsum string, prefix it by \"fancy:\"."
+                ))
             }?
             .rc(),
             T::Rearrange => {
                 expected_k_children(1)?;
-                Rearrange::from_string(children[0].clone(), &ps.extra, ps.name.clone())?.rc()
+                Rearrange::from_string(children[0].clone(), &ps.extra, ps.name)?.rc()
             }
             T::Symbol => {
                 expected_k_children(0)?;
@@ -870,9 +869,9 @@ impl Parser {
                     })?
                     .clone();
                 if ps.extra.is_empty() {
-                    Symbol::new_with_none_uuid(shape, ps.name.clone()).rc()
+                    Symbol::new_with_none_uuid(shape, ps.name).rc()
                 } else if ps.extra == "rand" {
-                    Symbol::new_with_random_uuid(shape, ps.name.clone()).rc()
+                    Symbol::new_with_random_uuid(shape, ps.name).rc()
                 } else {
                     Symbol::nrc(
                         shape,
@@ -880,19 +879,19 @@ impl Parser {
                             string: ps.extra.to_owned(),
                             err_msg: e.to_string(),
                         })?,
-                        ps.name.clone(),
+                        ps.name,
                     )
                 }
             }
             T::GeneralFunction => {
-                GeneralFunction::new_from_parse(children, ps.extra.clone(), ps.name.clone())?.rc()
+                GeneralFunction::new_from_parse(children, ps.extra.clone(), ps.name)?.rc()
             }
             T::Index => {
                 expected_k_children(1)?;
                 Index::try_new(
                     children[0].clone(),
                     TensorIndex::from_bijection_string(&ps.extra, tensor_cache)?,
-                    ps.name.clone(),
+                    ps.name,
                 )?
                 .rc()
             }
@@ -907,16 +906,14 @@ impl Parser {
                             variant: ps.variant,
                         })?
                         .clone(),
-                    ps.name.clone(),
+                    ps.name,
                 )?
                 .rc()
             }
             T::Module => {
                 extra_should_be_empty()?;
                 if children.len() == 0 {
-                    bail!(ParseCircuitError::ModuleNoSpecCircuit {
-                        name: ps.name.clone(),
-                    })
+                    bail!(ParseCircuitError::ModuleNoSpecCircuit { name: ps.name })
                 }
 
                 let nodes = children.split_off(1);
@@ -933,7 +930,7 @@ impl Parser {
                         self.module_check_unique_arg_names,
                     )
                     .context("spec construction failed in parse")?,
-                    ps.name.clone(),
+                    ps.name,
                 )
                 .context("module construction failed in parse")?
                 .rc()
@@ -945,16 +942,15 @@ impl Parser {
                         string: ps.extra.clone(),
                         err_msg: e.to_string(),
                     })
-                    .map(|uuid| Tag::nrc(children[0].clone(), uuid, ps.name.clone()))?
+                    .map(|uuid| Tag::nrc(children[0].clone(), uuid, ps.name))?
             }
             T::Cumulant => {
                 extra_should_be_empty()?;
-                Cumulant::nrc(children, ps.name.clone())
+                Cumulant::nrc(children, ps.name)
             }
             T::DiscreteVar => {
                 expected_k_children(2)?;
-                DiscreteVar::try_new(children[0].clone(), children[1].clone(), ps.name.clone())?
-                    .rc()
+                DiscreteVar::try_new(children[0].clone(), children[1].clone(), ps.name)?.rc()
             }
             T::StoredCumulantVar => {
                 let (cum_nums, uuid) =
@@ -982,12 +978,7 @@ impl Parser {
                         ident: ident.clone()
                     })
                 } else {
-                    StoredCumulantVar::try_new(
-                        zip(keys, children).collect(),
-                        uuid,
-                        ps.name.clone(),
-                    )?
-                    .rc()
+                    StoredCumulantVar::try_new(zip(keys, children).collect(), uuid, ps.name)?.rc()
                 }
             }
             T::SetSymbolicShape => {
@@ -1001,7 +992,7 @@ impl Parser {
                             variant: ps.variant,
                         })?
                         .clone(),
-                    ps.name.clone(),
+                    ps.name,
                 )?
                 .rc()
             }
@@ -1058,7 +1049,7 @@ pub enum ReferenceCircError {
     ByNameHasNoneName { circuit: CircuitRc },
 
     #[error("dup_idents={dup_idents:?} ({e_name})")]
-    DuplicateIdentifier { dup_idents: HashMap<String, usize> },
+    DuplicateIdentifier { dup_idents: HashMap<Name, usize> },
 }
 
 const REGEX_101_MSG: &str = "\nTry using regex101.com : )\n(TODO: auto generate regex test page via https://github.com/firasdib/Regex101/wiki/API#python-3)";
@@ -1186,7 +1177,7 @@ pub enum ParseCircuitError {
     #[error(
         "Module has no children, but should have at least a spec circuit name={name:?} ({e_name})"
     )]
-    ModuleNoSpecCircuit { name: Option<String> },
+    ModuleNoSpecCircuit { name: Option<Name> },
 
     #[error(
         "for module_ident={module_ident:?} the input child_ident={child_ident:?} {}\n{}\n{}\n({e_name})",
