@@ -7,7 +7,7 @@ use anyhow::{anyhow, bail, Context, Result};
 use itertools::Itertools;
 use pyo3::prelude::*;
 use rand::Rng;
-use rr_util::{fn_struct, py_types::MaybeNotSet, simple_default};
+use rr_util::{fn_struct, name::Name, py_types::MaybeNotSet, simple_default};
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 
 use crate::{
@@ -107,7 +107,11 @@ impl PrintHtmlOptions {
                 parent.as_module().map(|m| {
                     child_idx.checked_sub(1).map(|i| {
                         let arg_spec = &m.spec.arg_specs[i];
-                        arg_spec.symbol.name().map(|n| write!(desc, " ! '{}'", n));
+                        arg_spec
+                            .symbol
+                            .info()
+                            .name
+                            .map(|n| write!(desc, " ! '{}'", n));
                     })
                 });
             });
@@ -122,7 +126,7 @@ impl PrintHtmlOptions {
                 None => selfy.primary_color.clone(),
             };
             make_str_css_safe(&mut color)?;
-            let mut name = circ.name_cloned().unwrap_or("".to_owned());
+            let mut name = Name::string_maybe_empty(circ.info().name);
             make_str_safe(&mut name)?;
             write!(
                 nodes_string,
@@ -446,7 +450,7 @@ impl PrintHtmlOptions {
 
     pub fn repr_line(&self, circ: CircuitRc) -> Result<String> {
         let line_info = self.get_print_options().repr_line_info(circ.clone());
-        if let Some(name) = circ.name() {
+        if let Some(name) = circ.info().name {
             Ok(format!("'{name}'{}", &line_info?))
         } else {
             line_info
