@@ -14,7 +14,9 @@ use itertools::Itertools;
 use macro_rules_attribute::apply;
 use pyo3::{exceptions::PyValueError, prelude::*};
 use rr_util::{
+    compact_data::U8Set,
     filter_by_variant,
+    name::Name,
     opt_einsum::optimize_einsum_spec_cached,
     py_types::PY_CIRCUIT_ITEMS,
     python_error_exception,
@@ -56,7 +58,7 @@ pub fn add_flatten_once(add: &Add) -> Option<Add> {
     if !did_anything {
         return None;
     }
-    Some(Add::new(new_operands, add.name_cloned()))
+    Some(Add::new(new_operands, add.info().name))
 }
 
 #[pyfunction]
@@ -72,7 +74,7 @@ pub fn add_collapse_scalar_inputs(add: &Add) -> Option<Add> {
             non_scalar_inputs.iter().map(|x| x.clone().into()).collect();
         let new_scalar_input = Scalar::new(new_scalar, add.info().shape.clone(), None).rc();
         new_inputs.push(new_scalar_input);
-        Some(Add::new(new_inputs, add.name_cloned()))
+        Some(Add::new(new_inputs, add.info().name))
     }
 }
 
@@ -97,7 +99,7 @@ pub fn add_elim_zeros(add: &Add) -> Option<Add> {
     if !did_anything {
         None
     } else {
-        Some(Add::new(new_operands, add.name_cloned()))
+        Some(Add::new(new_operands, add.info().name))
     }
 }
 
@@ -117,22 +119,27 @@ pub fn add_deduplicate(add: &Add) -> Option<Add> {
             .map(|(hash, node)| {
                 let count = duplicate_counts[&hash];
                 if count > 1 {
-                    let node_count_name = node.name().map(|s| format!("{}_mul_count", s));
-                    let count_name = node.name().is_some().then(|| "count_from_dedup".to_owned());
+                    let node_count_name =
+                        node.info().name.map(|s| format!("{}_mul_count", s).into());
+                    let count_name = node
+                        .info()
+                        .name
+                        .is_some()
+                        .then(|| "count_from_dedup".into());
                     Einsum::scalar_mul(node, count as f64, node_count_name, count_name).rc()
                 } else {
                     node
                 }
             })
             .collect();
-        Some(Add::new(deduped_inputs, add.name_cloned()))
+        Some(Add::new(deduped_inputs, add.info().name))
     }
 }
 
 #[pyfunction]
 pub fn remove_add_few_input(add: &Add) -> Option<CircuitRc> {
     if add.nodes.is_empty() {
-        Some(Scalar::new(0f64, add.info().shape.clone(), add.name_cloned()).rc())
+        Some(Scalar::new(0f64, add.info().shape.clone(), add.info().name).rc())
     } else if add.nodes.len() == 1 {
         Some(add.nodes[0].clone())
     } else {
@@ -466,7 +473,7 @@ pub fn einsum_elim_identity(einsum: &Einsum) -> Option<CircuitRc> {
     {
         Some(einsum.args[0].0.clone())
     } else if einsum.args.is_empty() {
-        Some(Scalar::new(1.0, sv![], einsum.name_cloned()).rc())
+        Some(Scalar::new(1.0, sv![], einsum.info().name).rc())
     } else {
         None
     }
@@ -474,14 +481,10 @@ pub fn einsum_elim_identity(einsum: &Einsum) -> Option<CircuitRc> {
 
 #[pyfunction]
 pub fn index_merge_scalar(index: &Index) -> Option<CircuitRc> {
-    index.node.as_scalar().map(|scalar| {
-        Scalar::new(
-            scalar.value,
-            index.info().shape.clone(),
-            index.name_cloned(),
-        )
-        .rc()
-    })
+    index
+        .node
+        .as_scalar()
+        .map(|scalar| Scalar::new(scalar.value, index.info().shape.clone(), index.info().name).rc())
 }
 
 #[pyfunction]
@@ -499,9 +502,9 @@ pub fn index_fuse(index: &Index) -> Option<Index> {
         Some(Index::new(
             inner.node.clone(),
             compose(&index.index, &inner.index)?,
-            index.name_cloned().map(|n| {
+            index.info().name.map(|n| {
                 if n.ends_with(" idx idx") {
-                    n.strip_suffix(" idx").unwrap().to_owned()
+                    n.strip_suffix(" idx").unwrap().into()
                 } else {
                     n
                 }
@@ -516,7 +519,7 @@ pub fn rearrange_merge_scalar(rearrange: &Rearrange) -> Option<CircuitRc> {
         Scalar::new(
             scalar.value,
             rearrange.info().shape.clone(),
-            rearrange.name_cloned(),
+            rearrange.info().name,
         )
         .rc()
     })
@@ -546,7 +549,7 @@ pub fn concat_merge_uniform(concat: &Concat) -> Option<CircuitRc> {
             Scalar::new(
                 maybe_scalars[0].unwrap(),
                 concat.info().shape.clone(),
-                concat.name_cloned(),
+                concat.info().name,
             )
             .rc(),
         )
@@ -594,8 +597,8 @@ pub fn remove_axes_gen(
     if axes.is_empty() {
         return Some(circuit.clone());
     }
-    fn get_name<T: CircuitNode>(t: &T) -> Option<String> {
-        t.name_cloned().map(|x| format!("{}_rem_ax", x))
+    fn get_name<T: CircuitNode>(t: &T) -> Option<Name> {
+        t.info().name.map(|x| format!("{}_rem_ax", x).into())
     }
     match &***circuit {
         Circuit::Scalar(scalar) => Some(
@@ -665,7 +668,7 @@ pub fn generalfunction_pull_removable_axes(node: &GeneralFunction) -> Option<Cir
     let new_generalfunction = GeneralFunction::new(
         vec![remove_axes(&node.nodes[0], &removable_batchable_inp).unwrap()],
         node.spec.clone(),
-        node.name_cloned(),
+        node.info().name,
     );
     let removable_batchable_out: HashSet<usize> = removable_batchable_inp
         .iter()
@@ -702,7 +705,7 @@ pub fn concat_pull_removable_axes(node: &Concat) -> Option<CircuitRc> {
             .map(|node| remove_axes(node, &removable_non_axis).unwrap())
             .collect(),
         new_axis,
-        node.name_cloned(),
+        node.info().name,
     );
     Some(Rearrange::nrc_elim_identity(
         new_node.rc(),
@@ -733,7 +736,7 @@ pub fn einsum_pull_removable_axes(einsum: &Einsum) -> Option<CircuitRc> {
     if !did_anything {
         return None;
     }
-    let ints_in_new_args: HashSet<u8> = new_args
+    let ints_in_new_args: U8Set = new_args
         .iter()
         .flat_map(|(_node, ints)| ints.as_slice())
         .cloned()
@@ -743,9 +746,9 @@ pub fn einsum_pull_removable_axes(einsum: &Einsum) -> Option<CircuitRc> {
     let shape_map = einsum.shape_map().unwrap();
     let reduced_axes = einsum.reduced_axes();
     let scalar_mul: f64 = reduced_axes
-        .iter()
-        .filter(|i| !ints_in_new_args.contains(i))
-        .map(|i| shape_map[i] as f64)
+        .into_iter()
+        .filter(|i| !ints_in_new_args.contains(*i))
+        .map(|i| shape_map[&i] as f64)
         .product();
     if scalar_mul != 1.0 {
         new_args.push((Scalar::new(scalar_mul, sv![], None).rc(), tu8v![]));
@@ -754,10 +757,10 @@ pub fn einsum_pull_removable_axes(einsum: &Einsum) -> Option<CircuitRc> {
     let new_out_ints: EinsumAxes = einsum
         .out_axes
         .iter()
-        .filter(|i| ints_in_new_args.contains(i))
+        .filter(|i| ints_in_new_args.contains(**i))
         .copied()
         .collect();
-    let new_einsum = Einsum::nrc(new_args, new_out_ints.clone(), einsum.name_cloned());
+    let new_einsum = Einsum::nrc(new_args, new_out_ints.clone(), einsum.info().name);
     if new_out_ints[..] == einsum.out_axes[..] {
         Some(new_einsum)
     } else {
@@ -874,7 +877,7 @@ pub fn add_pull_removable_axes(add: &Add, remove_non_common_axes: bool) -> Optio
             }
         })
         .collect();
-    let add_circuit = Add::nrc(new_operands, add.name_cloned());
+    let add_circuit = Add::nrc(new_operands, add.info().name);
     Some(Rearrange::nrc_elim_identity(
         add_circuit,
         post_rearrange_spec,
@@ -900,7 +903,7 @@ pub fn add_make_broadcasts_explicit(add: &Add) -> Option<Add> {
     if !did_anything {
         return None;
     }
-    Some(Add::new(new_nodes, add.name_cloned()))
+    Some(Add::new(new_nodes, add.info().name))
 }
 
 pub fn distribute_once_raw(
@@ -916,7 +919,11 @@ pub fn distribute_once_raw(
             let suffix = suffix.unwrap_or("");
             let prefix = prefix.unwrap_or("");
 
-            einsum.rename(node.name().map(|n_s| format!("{prefix}{n_s}{suffix}")))
+            einsum.rename(
+                node.info()
+                    .name
+                    .map(|n_s| format!("{prefix}{n_s}{suffix}").into()),
+            )
         } else {
             einsum
         };
@@ -951,7 +958,7 @@ pub fn distribute_once_raw(
                     )
                 })
                 .collect::<Result<Vec<CircuitRc>>>()?;
-            Ok(Add::new(summands, einsum.name_cloned()))
+            Ok(Add::new(summands, einsum.info().name))
         }
         None => bail!(DistributeError::OperandIsNotAdd {
             einsum: einsum.clone(),
@@ -1041,7 +1048,7 @@ pub fn einsum_of_permute_merge(einsum: &Einsum) -> Option<Einsum> {
     Some(Einsum::new(
         new_args,
         einsum.out_axes.clone(),
-        einsum.name_cloned(),
+        einsum.info().name,
     ))
 }
 
@@ -1055,7 +1062,7 @@ pub fn permute_of_einsum_merge(rearrange: &Rearrange) -> Option<Einsum> {
                     .iter()
                     .map(|i| einsum.out_axes[*i])
                     .collect(),
-                rearrange.name_cloned(),
+                rearrange.info().name,
             )
         })
     })
@@ -1070,7 +1077,7 @@ pub fn einsum_elim_zero(einsum: &Einsum) -> Option<Scalar> {
         Some(Scalar::new(
             0.0,
             einsum.info().shape.clone(),
-            einsum.name_cloned(),
+            einsum.info().name,
         ))
     } else {
         None
@@ -1223,7 +1230,7 @@ pub fn rearrange_fuse(node: &Rearrange) -> Option<Rearrange> {
             Rearrange::new(
                 inner.node.clone(),
                 spec.canonicalize(true),
-                node.name_cloned(),
+                node.info().name,
             )
         })
         .ok()
@@ -1277,24 +1284,25 @@ pub fn push_down_index_raw(
     let mut on_sub = |child_idx,
                       node: CircuitRc,
                       index,
-                      force_name: Option<Option<String>>|
+                      force_name: Option<Option<Name>>|
      -> Result<CircuitRc> {
-        let new_name = force_name.unwrap_or_else(|| {
+        let new_name: Option<Name> = force_name.unwrap_or_else(|| {
             suffix
                 .as_ref()
-                .map(|s| node.name().map(|n_s| format!("{n_s}{s}")))
+                .map(|s| node.info().name.map(|n_s| format!("{n_s}{s}").into()))
                 .flatten()
         });
         call_on_sub(child_idx, Index::new(node, index, new_name))
     };
-    let out_name = node.name_cloned();
+    let out_name = node.info().name;
     let partial_name = || {
         suffix
             .as_ref()
             .map(|s| {
                 node.node
-                    .name()
-                    .map(|inner_name| format!("{inner_name}_partial_{s}"))
+                    .info()
+                    .name
+                    .map(|inner_name| format!("{inner_name}_partial_{s}").into())
             })
             .flatten()
     };
@@ -1817,7 +1825,7 @@ pub fn concat_repeat_to_rearrange(concat: &Concat) -> Option<Concat> {
     if !did_anything {
         return None;
     }
-    Some(Concat::new(new_nodes, concat.axis, concat.name_cloned()))
+    Some(Concat::new(new_nodes, concat.axis, concat.info().name))
 }
 
 #[test]
@@ -1923,7 +1931,7 @@ pub fn extract_add(add: &Add, sub: &Add) -> Option<Add> {
     }
     *counts.entry(sub.crc()).or_insert(0) += 1;
 
-    Some(Add::try_from_counts(&counts, add.name_cloned()).unwrap())
+    Some(Add::try_from_counts(&counts, add.info().name).unwrap())
 }
 
 /// De-duplicate children of Add nodes that are the same except for a scalar product
@@ -1935,12 +1943,12 @@ pub fn add_fuse_scalar_multiples(add: &Add) -> Option<Add> {
 
     let mut reduced_any = 0;
 
-    let scalars_and_children: Vec<(Einsum, f64, Option<String>)> = add
+    let scalars_and_children: Vec<(Einsum, f64, Option<Name>)> = add
         .children()
         .map(|c| match &**c {
             Circuit::Einsum(e) => {
                 let mut scalar: f64 = 1.0;
-                let mut new_name: Option<String> = None;
+                let mut new_name: Option<Name> = None;
                 let no_scalar_children: EinsumArgs = e
                     .args
                     .iter()
@@ -1949,13 +1957,13 @@ pub fn add_fuse_scalar_multiples(add: &Add) -> Option<Add> {
                         if let Some(x) = child.as_scalar() && x.info().shape.len() == 0  {
                             scalar *= x.value;
                             reduced_any += 1;
-                            if x.name().is_some() {
+                            if x.info().name.is_some() {
                                 if let Some(new_name_string) = &new_name {
                                     new_name = Some(
-                                        new_name_string.clone() + " + " + x.name().unwrap(),
+                                       ( new_name_string.string() + " + " + &x.info().name.unwrap()).into()
                                     );
                                 } else {
-                                    new_name = x.name_cloned();
+                                    new_name = x.info().name;
                                 }
                             }
                             false
@@ -1967,14 +1975,14 @@ pub fn add_fuse_scalar_multiples(add: &Add) -> Option<Add> {
                     .collect();
 
                 (
-                    Einsum::new(no_scalar_children, e.out_axes.clone(), e.name_cloned()),
+                    Einsum::new(no_scalar_children, e.out_axes.clone(), e.info().name),
                     scalar,
                     new_name,
                 )
             }
             Circuit::Scalar(s) => {
                 if s.info().shape.len() == 0 {
-                    (Einsum::empty(None), s.value, s.name_cloned())
+                    (Einsum::empty(None), s.value, s.info().name)
                 } else {
                     (Einsum::identity(c, None), 1.0, None)
                 }
@@ -1987,7 +1995,7 @@ pub fn add_fuse_scalar_multiples(add: &Add) -> Option<Add> {
         return None;
     }
 
-    let mut hm: HashMap<Einsum, (f64, Option<String>)> = HashMap::default();
+    let mut hm: HashMap<Einsum, (f64, Option<Name>)> = HashMap::default();
     for (einsum, scalar, name) in scalars_and_children {
         let (hm_val, hm_name) = hm.entry(einsum).or_insert((0.0, None));
         *hm_val += scalar;
@@ -2006,7 +2014,7 @@ pub fn add_fuse_scalar_multiples(add: &Add) -> Option<Add> {
             if scalar == 1.0 {
                 einsum.rc()
             } else {
-                let name = einsum.name_cloned();
+                let name = einsum.info().name;
                 let out_axes = einsum.out_axes;
                 let mut v = einsum.args;
                 v.push((Scalar::new(scalar, sv![], scalar_name).rc(), tu8v![]));
@@ -2015,7 +2023,7 @@ pub fn add_fuse_scalar_multiples(add: &Add) -> Option<Add> {
         })
         .collect();
 
-    Some(Add::new(nodes, add.name_cloned()))
+    Some(Add::new(nodes, add.info().name))
 }
 
 #[pyfunction]
@@ -2035,7 +2043,7 @@ pub fn einsum_permute_to_rearrange(einsum: &Einsum) -> Option<Rearrange> {
             )
             .unwrap()
             .canonicalize(false),
-            einsum.name_cloned(),
+            einsum.info().name,
         ));
     }
     None
