@@ -219,10 +219,13 @@ pub fn batch_einsum(einsum: &Einsum, context: &mut OptimizationContext) -> Resul
         .map(|(i, (c, ints))| {
             (
                 set_named_axes(
-                    Symbol::new_with_random_uuid(c.info().shape.clone(), Some(i.to_string())),
+                    Symbol::new_with_random_uuid(
+                        c.info().shape.clone(),
+                        Some(i.to_string().into()),
+                    ),
                     ints.iter()
                         .enumerate()
-                        .map(|(i, int)| (i as u8, int.to_string()))
+                        .map(|(i, int)| (i as u8, int.to_string().into()))
                         .collect(),
                 )
                 .unwrap() // u8 don't have spaces
@@ -234,7 +237,7 @@ pub fn batch_einsum(einsum: &Einsum, context: &mut OptimizationContext) -> Resul
     let einsum_named_axes = Einsum::try_new(
         einsum_args_named_axes.clone(),
         einsum.out_axes.clone(),
-        einsum.name_cloned(),
+        einsum.info().name,
     )
     .unwrap();
     let nested_big =
@@ -262,7 +265,7 @@ pub fn batch_einsum(einsum: &Einsum, context: &mut OptimizationContext) -> Resul
     }
 
     let batch_int = ints_needed[0];
-    let batch_str = batch_int.to_string();
+    let batch_str = batch_int.to_string().into();
 
     let batch_int_len = einsum.shape_map().unwrap()[&(batch_int as u8)];
     let max_interm_mem = intermediates_above_mem
@@ -290,7 +293,7 @@ pub fn batch_einsum(einsum: &Einsum, context: &mut OptimizationContext) -> Resul
         .map(|c| {
             batch_to_concat(
                 c.clone(),
-                axis_of_name(c, &batch_str).unwrap(),
+                axis_of_name(c, batch_str).unwrap(),
                 batch_size,
                 false,
                 false,
