@@ -163,6 +163,7 @@ fn all_simp_fns() -> SimpFns {
             f_wrap!(einsum_flatten_once, "einsum_flatten"),
             f_wrap!(einsum_of_permute_merge),
             f_wrap!(einsum_merge_scalars),
+            f_wrap!(einsum_remove_one),
             l_wrap!(einsum_pull_removable_axes),
             f_wrap!(einsum_elim_removable_axes_weak),
             f_wrap!(einsum_permute_to_rearrange),
@@ -224,6 +225,7 @@ impl Default for SimpFnSubset {
             "einsum_elim_zero",
             "einsum_elim_identity",
             "einsum_of_permute_merge",
+            "einsum_remove_one", // we currently avoid merge (I think it isn't typically what you want)
             "einsum_elim_removable_axes_weak",
             "einsum_permute_to_rearrange",
             "index_elim_identity",
@@ -295,6 +297,7 @@ impl SimpFnSubset {
                 [
                     "add_elim_removable_axes_weak",
                     "einsum_elim_removable_axes_weak",
+                    "einsum_remove_one",
                 ]
                 .into_iter()
                 .map(|x| x.to_owned())
