@@ -7,8 +7,8 @@ use circuit_base::{
 };
 use pyo3::prelude::*;
 use rr_util::{
-    py_types::PyOpAtAxes, rearrange_spec::RearrangeSpec, tensor_util::right_align_shapes,
-    util::HashBytes,
+    name::Name, py_types::PyOpAtAxes, rearrange_spec::RearrangeSpec,
+    tensor_util::right_align_shapes, util::HashBytes,
 };
 use rustc_hash::FxHashMap as HashMap;
 
@@ -94,11 +94,11 @@ pub fn module_remove_unused_inputs_op(
             circuit: m.spec.circuit.clone(),
             arg_specs,
         },
-        m.name().map(|s| {
+        m.info().name.map(|s| {
             if add_suffix_on_remove_unused {
-                format!("{} rem_unused", s)
+                format!("{} rem_unused", s).into()
             } else {
-                s.to_owned()
+                s.into()
             }
         }),
     );
@@ -121,7 +121,7 @@ pub fn module_remove_unused_inputs_op(
             out.rank(),
         )
         .context("rank error in module remove unused")?;
-        Rearrange::nrc(out, spec, m.name_cloned())
+        Rearrange::nrc(out, spec, m.info().name)
     } else {
         out
     };
@@ -160,7 +160,7 @@ pub fn module_strip_args(m: &Module) -> Module {
             circuit: m.spec.circuit.clone(),
             arg_specs,
         },
-        m.name_cloned(),
+        m.info().name,
     )
 }
 
@@ -174,7 +174,7 @@ pub fn extract_rewrite_raw(
     circuit: CircuitRc,
     input_specs: Vec<(CircuitRc, ModuleArgSpec)>,
     prefix_to_strip: Option<String>,
-    module_name: Option<String>,
+    module_name: Option<Name>,
     check_all_inputs_used: bool,
     check_unique_arg_names: bool,
 ) -> Result<Module> {
@@ -186,9 +186,12 @@ pub fn extract_rewrite_raw(
     )?;
     if let Some(pref) = &prefix_to_strip {
         spec = spec.map_circuit_unwrap(|x| {
-            if let Some(name) = x.name() {
-                x.clone()
-                    .rename(Some(name.strip_prefix(pref).unwrap_or(name).to_owned()))
+            if let Some(name) = x.info().name {
+                x.clone().rename(Some(
+                    name.strip_prefix(pref)
+                        .map(|z| Name::new(z))
+                        .unwrap_or(name),
+                ))
             } else {
                 x
             }
@@ -209,7 +212,7 @@ pub fn extract_rewrite_raw(
 pub fn fuse_concat_modules(
     circuit: CircuitRc,
     modules: Vec<Module>,
-    name: Option<String>,
+    name: Option<Name>,
 ) -> Result<CircuitRc> {
     if modules.is_empty() {
         bail!(anyhow!("no modules specified"))
@@ -235,7 +238,7 @@ pub fn fuse_concat_modules(
                         })
                         .collect(),
                     spec.clone(),
-                    x.name_cloned(),
+                    x.info().name,
                 )),
                 1 => Ok(x.clone()),
                 _ => Err(anyhow!("module has batch rank > 1")),
