@@ -53,7 +53,7 @@ pub enum IterativeMatcherFromPy {
 pub enum IterativeMatcherData {
     Match(Matcher),
     Term(bool),
-    Filter(FilterIterativeMatcher),
+    Restrict(RestrictIterativeMatcher),
     Children(ChildrenMatcher),
     SpecCircuit(IterativeMatcherRc),
     NoModuleSpec(IterativeMatcherRc),
@@ -159,7 +159,7 @@ impl IterativeMatcherData {
         match self {
             Self::Match(_) => uuid!("f1d5e8ec-cba0-496f-883e-78dd5cdc3a49"),
             Self::Term(_) => uuid!("365bc6b4-cbc6-4d82-8e14-7505e1229eec"),
-            Self::Filter(_) => uuid!("bcc5ccaa-afbe-414d-88b4-b8eac8c93ece"),
+            Self::Restrict(_) => uuid!("bcc5ccaa-afbe-414d-88b4-b8eac8c93ece"),
             Self::Children(_) => uuid!("0081e9c9-7fdb-43e1-900e-f1adde28f23d"),
             Self::SpecCircuit(_) => uuid!("b728a8c3-0493-4ee5-96c2-71e4900fc0f8"),
             Self::NoModuleSpec(_) => uuid!("b728a8c3-0493-4ee5-96c2-71e4900fc0f8"),
@@ -176,7 +176,7 @@ impl IterativeMatcherData {
             Self::Match(x) => {
                 hasher.update(&x.hash());
             }
-            Self::Filter(FilterIterativeMatcher {
+            Self::Restrict(RestrictIterativeMatcher {
                 iterative_matcher,
                 term_if_matches,
                 start_depth,
@@ -230,7 +230,7 @@ impl IterativeMatcherData {
 
 /// Helper with some basic rules you may want to use to control your node matching iterations.
 #[derive(Clone, Debug)]
-pub struct FilterIterativeMatcher {
+pub struct RestrictIterativeMatcher {
     pub iterative_matcher: IterativeMatcherRc,
     ///if true, stops once it has found a match
     pub term_if_matches: bool,
@@ -243,7 +243,7 @@ pub struct FilterIterativeMatcher {
     pub depth: u32,
 }
 
-impl FilterIterativeMatcher {
+impl RestrictIterativeMatcher {
     /// Fancy constructor which supports range
     ///
     /// TODO: add support for a builder with defaults because this is really annoying...
@@ -343,7 +343,7 @@ impl FilterIterativeMatcher {
         };
 
         let updated = map_updated(updated, |new| {
-            IterativeMatcherData::Filter(FilterIterativeMatcher {
+            IterativeMatcherData::Restrict(RestrictIterativeMatcher {
                 iterative_matcher: new,
                 depth: new_depth,
                 ..self.clone()
@@ -577,7 +577,7 @@ impl IterativeMatcher {
         match &self.data {
             IterativeMatcherData::Match(m) => m.validate_matched(matched),
             IterativeMatcherData::Term(_) => Ok(()),
-            IterativeMatcherData::Filter(m) => m.iterative_matcher.validate_matched(matched),
+            IterativeMatcherData::Restrict(m) => m.iterative_matcher.validate_matched(matched),
             IterativeMatcherData::Children(_) => Ok(()), /* not sure what else you could do here */
             IterativeMatcherData::SpecCircuit(_) => Ok(()), /* not sure what else you could do here */
             IterativeMatcherData::NoModuleSpec(m) => m.validate_matched(matched),
@@ -628,7 +628,7 @@ impl IterativeMatcher {
             &IterativeMatcherData::Term(match_next) => {
                 IterateMatchResults::new_finished(match_next)
             }
-            IterativeMatcherData::Filter(filter) => filter.match_iterate(circuit)?,
+            IterativeMatcherData::Restrict(filter) => filter.match_iterate(circuit)?,
             IterativeMatcherData::Children(children_matcher) => {
                 children_matcher.match_iterate(circuit)?
             }
@@ -1101,15 +1101,14 @@ impl IterativeMatcher {
         end_depth: Option<u32>,
         term_early_at: MatcherRc,
     ) -> IterativeMatcher {
-        // TODO: flatten
-        IterativeMatcherData::Filter(FilterIterativeMatcher::new(
-            self_.clone().into(),
+        // DEPRECATED: use restrict instead
+        restrict(
+            self_.clone().rc(),
             term_if_matches,
             start_depth,
             end_depth,
             term_early_at,
-        ))
-        .into()
+        )
     }
 
     #[args(
@@ -1123,12 +1122,13 @@ impl IterativeMatcher {
         depth_slice: Slice,
         term_early_at: MatcherRc,
     ) -> Result<IterativeMatcher> {
-        Ok(self_.filter(
+        // DEPRECATED: use restrict_sl instead
+        restrict_sl(
+            self_.clone().rc(),
             term_if_matches,
-            flip_op_result(depth_slice.start.map(|x| x.try_into()))?,
-            flip_op_result(depth_slice.stop.map(|x| x.try_into()))?,
+            depth_slice,
             term_early_at,
-        ))
+        )
     }
 
     #[args(rest = "*", must_be_sub = "false")]
@@ -1309,3 +1309,65 @@ pub enum IterativeMatcherError {
         num_children: usize,
     },
 }
+
+#[pyfunction(
+    term_if_matches = "false",
+    start_depth = "None",
+    end_depth = "None",
+    term_early_at = "MatcherFromPyBase::Always(false).into()"
+)]
+pub fn restrict(
+    matcher: IterativeMatcherRc,
+    term_if_matches: bool,
+    start_depth: Option<u32>,
+    end_depth: Option<u32>,
+    term_early_at: MatcherRc,
+) -> IterativeMatcher {
+    // TODO: flatten
+    IterativeMatcherData::Restrict(RestrictIterativeMatcher::new(
+        matcher.clone().into(),
+        term_if_matches,
+        start_depth,
+        end_depth,
+        term_early_at,
+    ))
+    .into()
+}
+
+#[pyfunction(
+    term_if_matches = "false",
+    depth_slice = "Slice::IDENT",
+    term_early_at = "MatcherFromPyBase::Always(false).into()"
+)]
+pub fn restrict_sl(
+    matcher: IterativeMatcherRc,
+    term_if_matches: bool,
+    depth_slice: Slice,
+    term_early_at: MatcherRc,
+) -> Result<IterativeMatcher> {
+    Ok(matcher.filter(
+        term_if_matches,
+        flip_op_result(depth_slice.start.map(|x| x.try_into()))?,
+        flip_op_result(depth_slice.stop.map(|x| x.try_into()))?,
+        term_early_at,
+    ))
+}
+
+#[pyfunction(
+    start_depth = "None",
+    end_depth = "None",
+    term_early_at = "MatcherFromPyBase::Always(false).into()"
+)]
+pub fn new_traversal(
+    start_depth: Option<u32>,
+    end_depth: Option<u32>,
+    term_early_at: MatcherRc,
+) -> IterativeMatcher {
+    restrict(
+        IterativeMatcher::noop_traversal().into(),
+        false,
+        start_depth,
+        end_depth,
+        term_early_at,
+    )
+}
