@@ -335,7 +335,7 @@ impl SampleSpecTrait for RandomSampleSpec {
         Scalar::new(
             1. / count_elements(&self.shape) as f64,
             self.shape.clone(),
-            Some("empirical_weights_for_sampled".to_owned()),
+            Some("empirical_weights_for_sampled".into()),
         )
         .rc()
     }
@@ -489,7 +489,7 @@ impl SampleSpecTrait for RunDiscreteVarAllSpec {
         all_weights
             .clone()
             .mul(
-                GeneralFunction::new_by_name(vec![all_weights_sum], "reciprocal".to_owned(), None)
+                GeneralFunction::new_by_name(vec![all_weights_sum], "reciprocal".into(), None)
                     .unwrap()
                     .rc(),
                 None,
@@ -705,7 +705,7 @@ impl Sampler {
     }
     fn normed_weights(&self) -> CircuitRc {
         let weight_sum_reciprocal =
-            GeneralFunction::new_by_name(vec![self.weights_sum()], "reciprocal".to_owned(), None)
+            GeneralFunction::new_by_name(vec![self.weights_sum()], "reciprocal".into(), None)
                 .unwrap()
                 .rc();
 
@@ -721,8 +721,9 @@ impl Sampler {
         let spec_nb_axes = self.spec_nb_axes() as u8;
         let name = self.suffix().and_then(|s| {
             circuit
-                .name()
-                .map(|c_name| format!("{}_{}_expectation", c_name, s))
+                .info()
+                .name
+                .map(|c_name| format!("{}_{}_expectation", c_name, s).into())
         });
         Ok(Einsum::try_new(
             vec![
