@@ -28,7 +28,9 @@ use rr_util::{
     cached_method,
     caching::FastUnboundedCache,
     eq_by_big_hash::EqByBigHash,
-    fn_struct, impl_eq_by_big_hash, pycall, python_error_exception,
+    fn_struct, impl_eq_by_big_hash,
+    name::Name,
+    pycall, python_error_exception,
     rearrange_spec::{OpSize, RearrangeSpec, RearrangeSpecError},
     sv, tu8v, unwrap,
     util::{DimNumMaker, HashBytes},
@@ -49,7 +51,7 @@ pub fn extract_rewrite(
     circuit: CircuitRc,
     matcher: IterativeMatcherRc,
     prefix_to_strip: Option<String>,
-    module_name: Option<String>,
+    module_name: Option<Name>,
     check_all_inputs_used: bool,
     check_unique_arg_names: bool,
     circuit_to_arg_spec: Option<PyObject>,
@@ -66,7 +68,7 @@ pub fn extract_rewrite(
             .map(|z| (n, z))
         })
         .collect::<Result<Vec<_>>>()?;
-    specs.sort_by_key(|x| x.1.symbol.name_cloned());
+    specs.sort_by_key(|x| x.1.symbol.info().name);
     extract_rewrite_raw(
         circuit,
         specs,
@@ -143,7 +145,7 @@ pub fn module_new_bind(
     spec_circuit: CircuitRc,
     binders: Vec<Binder>,
     check_unique_arg_names: bool,
-    name: Option<String>,
+    name: Option<Name>,
 ) -> Result<Module> {
     let (nodes, arg_specs) = binders
         .into_iter()
@@ -291,7 +293,7 @@ pub fn fuse_modules_impl(
                         )
                         .unwrap();
 
-                        let rep_name = node.name().map(|x| format!("{} rep_fuse", x));
+                        let rep_name = node.info().name.map(|x| format!("{} rep_fuse", x).into());
                         Rearrange::nrc(node, spec, rep_name)
                     } else {
                         node
@@ -399,7 +401,7 @@ fn_struct!(
         running_circuit: CircuitRc,
         modules: Vec<Module>,
         pushed_overall_mod_count: Option<usize>
-    ) -> Option<String>
+    ) -> Option<Name>
 );
 
 #[pyfunction(bind_name = "\"bind\".to_owned()")]
@@ -407,22 +409,26 @@ pub fn default_nested_module_namer(bind_name: String) -> NestedModuleNamer {
     NestedModuleNamer::Dyn(NestedModuleNamerDynStruct(Arc::new(
         move |base_circuit, _, modules, _| {
             if modules.is_empty() {
-                return Ok(base_circuit.name_cloned());
+                return Ok(base_circuit.info().name);
             }
 
-            if base_circuit.name().is_none() || modules.iter().any(|m| m.name().is_none()) {
+            if base_circuit.info().name.is_none() || modules.iter().any(|m| m.info().name.is_none())
+            {
                 return Ok(None);
             }
 
-            Ok(Some(format!(
-                "{} {bind_name}:{}",
-                base_circuit.name().unwrap(),
-                modules
-                    .iter()
-                    .map(|x| x.name().unwrap())
-                    .collect::<Vec<_>>()
-                    .join(",")
-            )))
+            Ok(Some(
+                format!(
+                    "{} {bind_name}:{}",
+                    base_circuit.info().name.unwrap(),
+                    modules
+                        .iter()
+                        .map(|x| x.info().name.unwrap().string())
+                        .collect::<Vec<String>>()
+                        .join(",")
+                )
+                .into(),
+            ))
         },
     )))
 }
@@ -938,7 +944,7 @@ impl ModulePusher {
                             .copied()
                             .collect(),
                         arg_spec.symbol.uuid.clone(),
-                        arg_spec.symbol.name_cloned(),
+                        arg_spec.symbol.info().name,
                     );
 
                     let orig_sim = arg_spec.symbol.clone();
@@ -1151,7 +1157,7 @@ impl ModuleConstructUpdateBindings {
             let new_spec_circuit = self
                 .deep_remove_unused(m.spec.circuit.as_module_unwrap().clone(), current_count - 1)
                 .rc();
-            let name = m.name_cloned();
+            let name = m.info().name;
             Module::new(
                 m.nodes,
                 ModuleSpec {
@@ -1184,7 +1190,7 @@ impl ModuleConstructUpdateBindings {
                         circuit: new_spec,
                         arg_specs: m.spec.arg_specs.clone(),
                     },
-                    m.name_cloned(),
+                    m.info().name,
                 )
                 .context(
                     "module construction failed in update bindings after updating spec circuit",
@@ -1306,11 +1312,12 @@ pub fn default_update_bindings_nested_namer(
     NestedModuleNamer::Dyn(NestedModuleNamerDynStruct(Arc::new(
         move |base_circuit, running_circuit, modules, pushed_overall_mod_count| {
             if pushed_overall_mod_count.is_none() && keep_name_if_not_leaf {
-                Ok(base_circuit.name_cloned())
+                Ok(base_circuit.info().name)
             } else if pushed_overall_mod_count.is_none() && short_if_not_leaf {
                 Ok(base_circuit
-                    .name()
-                    .map(|name| format!("{name} {bind_name}")))
+                    .info()
+                    .name
+                    .map(|name| format!("{name} {bind_name}").into()))
             } else {
                 default_overall_namer.call(
                     base_circuit,
@@ -1483,7 +1490,7 @@ impl SymbolExtractor {
                         circuit: new_m.spec.circuit.clone(),
                         arg_specs,
                     },
-                    m.name().map(|s| format!("{} extracted", s)),
+                    m.info().name.map(|s| format!("{} extracted", s).into()),
                 )
             };
             let traversal_per_child = iter::once(spec_traversal)
@@ -1554,7 +1561,7 @@ pub fn extract_symbols(
     conform_batch_if_needed: bool,
     traversal: IterativeMatcherRc,
 ) -> Result<Module> {
-    let name = circuit.name_cloned();
+    let name = circuit.info().name;
     let (extracted_circ, extracted_syms) = SymbolExtractor {
         conform_batch_if_needed,
         ..Default::default()
