@@ -90,6 +90,10 @@ generalfunctions = {
     "min": lambda x: torch.min(x, dim=-1)[0],
     "max": lambda x: torch.max(x, dim=-1)[0],
     "last_dim_size": lambda x: torch.full(x.shape[:-1], x.shape[-1], dtype=x.dtype, device=x.device),
+    "abs": torch.abs,
+    "exp": torch.exp,
+    "log": torch.log,
+    "logit": torch.logit,
 }
 
 
