@@ -19,6 +19,7 @@ use uuid::uuid;
 use crate::{
     compact_data::{TinyVecU8, U8Set},
     make_single_many,
+    name::Name,
     py_types::{einops_repeat, PyOpAtAxes, Tensor, PY_CIRCUIT_ITEMS},
     python_error_exception, sv,
     symbolic_size::{SymbolicSizeProduct, SIZE_PROD_MATCH},
@@ -571,15 +572,15 @@ impl RearrangeSpec {
             .iter()
             .map(|v| if v.len() == 1 { Some(v[0]) } else { None })
             .collect::<Option<Vec<AxisInt>>>()?;
-        let mut axis_to_name: HashMap<AxisInt, &String> = Default::default();
+        let mut axis_to_name: HashMap<AxisInt, Name> = Default::default();
         for (i, a) in input_int_unpacked.iter().enumerate() {
             if let Some(name) = node_named_axes.get(&(i as u8)) {
-                axis_to_name.insert(*a, name);
+                axis_to_name.insert(*a, *name);
             }
         }
         for (i, a) in output_int_unpacked.iter().enumerate() {
             if let Some(name) = self_named_axes.get(&(i as u8)) {
-                axis_to_name.insert(*a, name);
+                axis_to_name.insert(*a, *name);
             }
         }
         if axis_to_name.len() > HashSet::from_iter(axis_to_name.values().cloned()).len() {
@@ -587,12 +588,12 @@ impl RearrangeSpec {
         }
         let inp_words = input_int_unpacked
             .iter()
-            .map(|a| Some((*axis_to_name.get(a)?).clone()))
+            .map(|a| Some((*axis_to_name.get(a)?).into()))
             .collect::<Option<Vec<String>>>()?
             .join(" ");
         let out_words = output_int_unpacked
             .iter()
-            .map(|a| Some((*axis_to_name.get(a)?).clone()))
+            .map(|a| Some((*axis_to_name.get(a)?).into()))
             .collect::<Option<Vec<String>>>()?
             .join(" ");
         Some(inp_words + " -> " + &out_words)
