@@ -14,13 +14,13 @@ use once_cell::sync::Lazy;
 use pyo3::{types::PyBytes, FromPyObject, IntoPy, PyObject, Python};
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 
-use crate::{compact_data::TinyVecU8, make_lazy, simple_from};
+use crate::{compact_data::TinyVecU8, make_lazy, name::Name, simple_from};
 pub type AxisInt = u8;
 pub type HashBytes = [u8; 32];
 // we could use u64 instead of u8 if we wanted.
 // this is u8, so we want to use multiple of 8 inline so we don't pad
 pub type EinsumAxes = TinyVecU8;
-pub type NamedAxes = BTreeMap<AxisInt, String>;
+pub type NamedAxes = BTreeMap<AxisInt, Name>;
 
 #[derive(FromPyObject, Copy, Clone, Debug, Eq, PartialEq, Ord, PartialOrd)]
 pub struct HashBytesToPy(pub HashBytes);
