@@ -244,7 +244,7 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         m
     )?)?;
     m.add_function(wrap_pyfunction!(
-        circuit_rewrites::compiler_strip::strip_names_py,
+        circuit_rewrites::compiler_strip::strip_names_and_tags_py,
         m
     )?)?;
     m.add_function(wrap_pyfunction!(deep_maybe_distribute_py, m)?)?;
@@ -379,6 +379,10 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         circuit_base::module::conform_all_modules,
         m
     )?)?;
+    m.add_function(wrap_pyfunction!(
+        circuit_base::module::inline_single_callsite_modules,
+        m
+    )?)?;
     m.add_function(wrap_pyfunction!(
         circuit_base::module::clear_module_circuit_caches,
         m
